#include <cstdio>
#include <cstdlib>
#include <cassert>

void freeArray(int** &array, int maxRows) {
    if(array) {
        for(int i = 0; i < maxRows && array[i]; ++i)
            delete[] array[i];
        delete[] array;
        array = 0;
    }
}

void printArray(int** array, int maxRows, int columns) {
    for(int i = 0; i < maxRows && array[i]; ++i) {
        for(int j = 0; j < columns; ++j)
            printf("%d", array[i][j]);
        printf("\n");
    }
}

int** createArray(int nrow, int ncol) {
    int** array = new int*[nrow];
    for(int i = 0; i < nrow; ++i){
        array[i] = new int[ncol];
        for(int j=0; j<ncol; ++j)
            array[i][j]=i+1;
    }
    return array;
}


int removeRows1(int **a, int *key, int nrow){
    int ip = 0, i=0; //i -   , ip -   
    while (i < nrow){
        while(!key[i] && i<nrow){ //  ,  
            delete[] a[i]; ++i; 
        }
        if(i == nrow) break; //    
        if (ip != i) a[ip] = a[i]; //    ,    
        ++ip; ++i; 
    }
    return ip;
}

int removeRows2(int** array, int *key, int maxRows) {
    int** temp = array;
    for(int i = 0; i < maxRows && array[i]; ++i) 
        if(!key[i]) {
            delete[] array[i];
            array[i] = 0;
        } else {
            if(temp != array + i) {
                *temp = array[i];
                array[i] = 0;
            }
            ++temp;
        }
        return temp - array;
}

int removeRows3(int** initMatrix, int *key, int rowsNumber) {
    int deletedRowsQuantity = 0;
    int *zeroCount = new int [rowsNumber];  // 
    for(int i=0; i<rowsNumber; ++i) zeroCount[i]=key[i]; // 
    for(int i = 0; i < rowsNumber; ++i)
    {
        if(!zeroCount[i])
        {
            ++deletedRowsQuantity;
            delete initMatrix[i]; 
        }
        if((deletedRowsQuantity != 0) && (i < rowsNumber-1))
        {
            initMatrix[i+1-deletedRowsQuantity] = initMatrix[i+1];
            zeroCount[i+1-deletedRowsQuantity] = zeroCount[i+1];
        }
    }
    delete [] zeroCount; // 
    return rowsNumber-deletedRowsQuantity;
}

int removeRows4(int** Matrix, int *key, int m) {
    int mm = m;	            //  
    int L = 0;              //""  ,    

    for (int i = 0; i < m; ++i){
        if (!key[i]){     // 
            --mm;           // 
        } else {
            if (L != i){ // "" ,     L
                delete[] Matrix[L];
                Matrix[L] = Matrix[i];
                Matrix[i] = 0; //              2- delete   
            }
            ++L;             // " "
        }
    }
    return mm;
}

int removeRows5(int** DynamicArray, int *key, int nrow) {
	int nextGood = 0;
	while (!key[nextGood]){						//search the first good row
		++nextGood;
	}
	int j =0;
	for (j = 0; nextGood < nrow; ){
		if (!key[j]) {							//delete i-th row, if it's bad
			delete []DynamicArray[j];
		}
		while (!key[nextGood] && nextGood < nrow){
			++nextGood;											//search next good row
		}
		if (nextGood != nrow && key[nextGood]){						//if it's not the end
			DynamicArray[j] = DynamicArray[nextGood];				//move pointer to the net good row
			++nextGood;												//this row is busy now
			++j;													//we've done with this pointer
		}
	}
	for (int k=j; k<nrow; k++){									//As the number of deleted rows is i,
		if (!key[j]) delete DynamicArray[k];					//we have to clear memory in last (n-i) rows.
	}															
	nrow = j;													//new number of rows
	return(nrow);
}

int removeRows6(int** arr, int *key, int nrow) {
    int counter=0;
    for (int i = 0; i < nrow; ++i)
    {
        if (!key[i])
        {
            delete[] arr[i];
            ++counter;
        }
        if (i + 1 < nrow)
        {
            arr[i - counter + 1] = arr[i + 1];
        }
    }
    return (nrow - counter);
}

int removeRows7(int** arr, int *key, int nrow){
	int prev = 0;
	int count = 0;
	for (int i = 0; i < nrow; ++i){
		while ((i < nrow) && (!key[i])){
			++count;
			delete[] arr[i];
			++i;
		}
		if (i < nrow){
			arr[prev] = arr[i];
			++prev; 
		}
	}
	return nrow-count;  //  
}

bool checkResult(int **a, int* goodRows, int iGood, int nrow)
{
    if(iGood != nrow) return false;
    for(int i=0; i<nrow; ++i){
        if(a[i][0]!=goodRows[i]) return false;
    }
    return true;
}

int testArray(int ** a, int goodRows[], int iGood, int nrow, int ncol, int num){
    int * key = new int[nrow];
    for(int i=0; i<nrow; ++i) key[i]=0;
    for(int i=0; i<iGood; ++i) key[goodRows[i]-1]=1;

    int newNrow = nrow;
    printf("\n Var. %d  ============\n", num);
    a=createArray(nrow, ncol);

    switch(num){
    case 1:	newNrow = removeRows1(a, key, nrow); break;
    case 2:	newNrow = removeRows2(a, key, nrow); break;
    case 3:	newNrow = removeRows3(a, key, nrow); break;
    case 4:	newNrow = removeRows4(a, key, nrow); break;
    case 5:	newNrow = removeRows5(a, key, nrow); break;
    case 6:	newNrow = removeRows6(a, key, nrow); break;
    case 7:	newNrow = removeRows7(a, key, nrow); break;
	}
    bool res = checkResult(a, goodRows, iGood, newNrow); 
    if(!res) printf("\n error in var %d\n", num);
    assert(res);
    printArray(a, newNrow, ncol);
    freeArray(a, newNrow);
    delete[] key;
    return res;
}

int main(){
    const int nrow=6, ncol=6;
    int newNrow;
    //int goodRows[]={1,2,5,6};
    //int goodRows[]={1,2,3,4,5,6};
    int goodRows[]={3,4};

    int iGood = sizeof(goodRows)/sizeof(int);

    int ** a = 0;

    testArray(a, goodRows, iGood, nrow, ncol, 1);
    testArray(a, goodRows, iGood, nrow, ncol, 2);
    testArray(a, goodRows, iGood, nrow, ncol, 3);
    testArray(a, goodRows, iGood, nrow, ncol, 4);
    testArray(a, goodRows, iGood, nrow, ncol, 5);
    testArray(a, goodRows, iGood, nrow, ncol, 6);
    testArray(a, goodRows, iGood, nrow, ncol, 7);

    system("pause");
}